/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                Java Language Certus Mini N SDK, Version 7.3                */
/*                    Copyright 2024, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2024 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* Extended Device Information */
public class ANPacket13 extends ANPacket3
{
	public static final int LENGTH = 36;

	public static final int DEVICE_SUBTYPE_ID_UNKNOWN = 255;

	public static final int DEVICE_SUBTYPE_ID_BOREAS_ACC_15G_V1 = 0;
	public static final int DEVICE_SUBTYPE_ID_BOREAS_ACC_15G_V2 = 1;
	public static final int DEVICE_SUBTYPE_ID_BOREAS_ACC_30G = 2;

	public static final int DEVICE_SUBTYPE_ID_CERTUS = 0;
	public static final int DEVICE_SUBTYPE_ID_CERTUS_EVO = 1;
	public static final int DEVICE_SUBTYPE_ID_CERTUS_OEM = 2;
	public static final int DEVICE_SUBTYPE_ID_CERTUS_EVO_OEM = 3;

	public static final int DEVICE_SUBTYPE_ID_CERTUS_MINI = 0;
	public static final int DEVICE_SUBTYPE_ID_CERTUS_MINI_OEM = 2;

	public long deviceSubType;

	public ANPacket13()
	{
		softwareVersion = 0;
		deviceID = 0;
		hardwareRevision = 0;
		serialNumber = new long[3];
		serialNumber[0] = 0;
		serialNumber[1] = 0;
		serialNumber[2] = 0;
		deviceSubType = DEVICE_SUBTYPE_ID_UNKNOWN;
	}

	public ANPacket13(ANPacket packet)
	{
		softwareVersion = TypeConversion.bytesToUInt32(packet.data, 0);
		deviceID = TypeConversion.bytesToUInt32(packet.data, 4);
		hardwareRevision = TypeConversion.bytesToUInt32(packet.data, 8);
		serialNumber = new long[3];
		serialNumber[0] = TypeConversion.bytesToUInt32(packet.data, 12);
		serialNumber[1] = TypeConversion.bytesToUInt32(packet.data, 16);
		serialNumber[2] = TypeConversion.bytesToUInt32(packet.data, 20);
		deviceSubType = TypeConversion.bytesToUInt32(packet.data, 24);
	}

	@Override
	public String deviceIdString()
	{
		String deviceIdString = super.deviceIdString();
		String subTypeString = subTypeString();

		if (!subTypeString.isEmpty())
		{
			switch ((int) deviceID)
			{
				case DEVICE_ID_CERTUS:
				case DEVICE_ID_CERTUS_MINI_A:
				case DEVICE_ID_CERTUS_MINI_D:
				case DEVICE_ID_CERTUS_MINI_N:
					deviceIdString += " " + subTypeString;
					break;
			}
		}
		return deviceIdString;
	}

	public String subTypeString()
	{
		return subTypeString((int) deviceID, (int) deviceSubType);
	}

	public static String subTypeString(int deviceId, int deviceSubType)
	{
		String descriptor = "";
		switch (deviceId)
		{
			case DEVICE_ID_CERTUS:
				switch (deviceSubType)
				{
					case DEVICE_SUBTYPE_ID_CERTUS_EVO:
						descriptor = "Evo";
						break;
					case DEVICE_SUBTYPE_ID_CERTUS_OEM:
						descriptor = "OEM";
						break;
					case DEVICE_SUBTYPE_ID_CERTUS_EVO_OEM:
						descriptor = "Evo OEM";
						break;
					default:
						break;
				}
				break;
			case DEVICE_ID_BOREAS_A70:
			case DEVICE_ID_BOREAS_D70:
			case DEVICE_ID_BOREAS_A90:
			case DEVICE_ID_BOREAS_D90:
				switch (deviceSubType)
				{
					case DEVICE_SUBTYPE_ID_BOREAS_ACC_15G_V1:
						descriptor = "15G v1 Accelerometer";
						break;
					case DEVICE_SUBTYPE_ID_BOREAS_ACC_15G_V2:
						descriptor = "15G v2 Accelerometer";
						break;
					case DEVICE_SUBTYPE_ID_BOREAS_ACC_30G:
						descriptor = "30G Accelerometer";
						break;
					default:
						break;
				}
				break;
			case DEVICE_ID_CERTUS_MINI_A:
			case DEVICE_ID_CERTUS_MINI_D:
			case DEVICE_ID_CERTUS_MINI_N:
				switch (deviceSubType)
				{
					case DEVICE_SUBTYPE_ID_CERTUS_MINI_OEM:
						descriptor = "OEM";
						break;
					default:
						break;
				}
				break;
			default:
				break;
		}
		return descriptor;
	}
}
