/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                Java Language Certus Mini N SDK, Version 7.3                */
/*                    Copyright 2024, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2024 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* GNSS Configuration */
public class ANPacket197
{
	public static final int LENGTH = 85;

	public boolean gpsL1caEnabled;
	public boolean gpsL1cEnabled;
	public boolean gpsL1pEnabled;
	public boolean gpsL2cEnabled;
	public boolean gpsL2pEnabled;
	public boolean gpsL2mEnabled;
	public boolean gpsL5Enabled;
	public boolean glonassG1caEnabled;
	public boolean glonassG1pEnabled;
	public boolean glonassL1ocEnabled;
	public boolean glonassL1scEnabled;
	public boolean glonassG2caEnabled;
	public boolean glonassG2pEnabled;
	public boolean glonassL2ocEnabled;
	public boolean glonassL2scEnabled;
	public boolean glonassL3ocEnabled;
	public boolean glonassL3scEnabled;
	public boolean beidouB1Enabled;
	public boolean beidouB2Enabled;
	public boolean beidouB3Enabled;
	public boolean galileoE1Enabled;
	public boolean galileoE5aEnabled;
	public boolean galileoE5bEnabled;
	public boolean galileoE5abEnabled;
	public boolean galileoE6Enabled;
	public boolean qzssL1caEnabled;
	public boolean qzssL1saifEnabled;
	public boolean qzssL1cEnabled;
	public boolean qzssL2cEnabled;
	public boolean qzssL5Enabled;
	public boolean qzssLexEnabled;
	public boolean sbasL1caEnabled;
	public long frequenciesCode;

	public float pdopMask;
	public float tdopMask;
	public int elevationMask;
	public int snrMask;
	public boolean sbasCorrectionsEnabled;
	public int lbandMode;
	public long lbandFrequency;
	public long lbandBaud;
	public long primaryAntennaType;
	public long secondaryAntennaType;
	public int lbandSatelliteId;

	private int[] lBandSatelliteNumbers = {
		110,
		100,
		2,
		3,
		4,
		5,
		6,
		7,
		8,
		9,
		10,
		11,
		12,
		13,
		14,
		16,
		18,
		19,
		20,
		21,
		22,
		23,
		24,
		25,
		26,
		27,
		28,
		30,
		31
		};
	private String[] lBandSatelliteNames = {
		"Auto",
		"Custom",
		"IOSAT",
		"IOR",
		"SASAT",
		"POR",
		"EASAT",
		"IRSAT",
		"AORET",
		"ESAT",
		"AORE",
		"AORW",
		"ASAT",
		"ARSAT",
		"AUSAT",
		"AMSAT",
		"OCSAT",
		"RTXAL",
		"ERSAT",
		"RTXEU",
		"RTXEN",
		"RTXNA",
		"RTXAF",
		"RTXIR",
		"RTXME",
		"RTXPA",
		"RTXAP",
		"RTXSA",
		"RTXCN"
		};
	private int[] lBandModeNumbers = {
		0,
		2,
		3,
		4,
		5,
		8,
		9,
		10,
		11,
		13,
		14,
		15,
		16
		};
	private String[] lBandModeNames = {
		"Disabled",
		"Omnistar Auto",
		"Omnistar HP",
		"Omnistar XP",
		"Omnistar VBS",
		"Omnistar G2",
		"Omnistar HP+G2",
		"Omnistar HP+XP",
		"Trimble RTX",
		"Omnistar L1 Only",
		"Omnistar G4 Only",
		"Omnistar G2+ Only",
		"Omnistar G4+ Only"
		};

	public ANPacket197()
	{
		gpsL1caEnabled = false;
		gpsL1cEnabled = false;
		gpsL1pEnabled = false;
		gpsL2cEnabled = false;
		gpsL2pEnabled = false;
		gpsL2mEnabled = false;
		gpsL5Enabled = false;
		glonassG1caEnabled = false;
		glonassG1pEnabled = false;
		glonassL1ocEnabled = false;
		glonassL1scEnabled = false;
		glonassG2caEnabled = false;
		glonassG2pEnabled = false;
		glonassL2ocEnabled = false;
		glonassL2scEnabled = false;
		glonassL3ocEnabled = false;
		glonassL3scEnabled = false;
		beidouB1Enabled = false;
		beidouB2Enabled = false;
		beidouB3Enabled = false;
		galileoE1Enabled = false;
		galileoE5aEnabled = false;
		galileoE5bEnabled = false;
		galileoE5abEnabled = false;
		galileoE6Enabled = false;
		qzssL1caEnabled = false;
		qzssL1saifEnabled = false;
		qzssL1cEnabled = false;
		qzssL2cEnabled = false;
		qzssL5Enabled = false;
		qzssLexEnabled = false;
		sbasL1caEnabled = false;
		sbasCorrectionsEnabled = false;
	}

	public ANPacket197(ANPacket packet)
	{
		 gpsL1caEnabled = (packet.data[1] & 0x01) > 0;
		 gpsL1cEnabled = (packet.data[1] & 0x02) > 0;
		 gpsL1pEnabled = (packet.data[1] & 0x04) > 0;
		 gpsL2cEnabled = (packet.data[1] & 0x08) > 0;
		 gpsL2pEnabled = (packet.data[1] & 0x10) > 0;
		 gpsL2mEnabled = (packet.data[1] & 0x20) > 0;
		 gpsL5Enabled = (packet.data[1] & 0x40) > 0;
		 glonassG1caEnabled = (packet.data[1] & 0x80) > 0;
		 glonassG1pEnabled = (packet.data[2] & 0x01) > 0;
		 glonassL1ocEnabled = (packet.data[2] & 0x02) > 0;
		 glonassL1scEnabled = (packet.data[2] & 0x04) > 0;
		 glonassG2caEnabled = (packet.data[2] & 0x08) > 0;
		 glonassG2pEnabled = (packet.data[2] & 0x10) > 0;
		 glonassL2ocEnabled = (packet.data[2] & 0x20) > 0;
		 glonassL2scEnabled = (packet.data[2] & 0x40) > 0;
		 glonassL3ocEnabled = (packet.data[2] & 0x80) > 0;
		 glonassL3scEnabled = (packet.data[3] & 0x01) > 0;
		 beidouB1Enabled = (packet.data[3] & 0x02) > 0;
		 beidouB2Enabled = (packet.data[3] & 0x04) > 0;
		 beidouB3Enabled = (packet.data[3] & 0x08) > 0;
		 galileoE1Enabled = (packet.data[3] & 0x10) > 0;
		 galileoE5aEnabled = (packet.data[3] & 0x20) > 0;
		 galileoE5bEnabled = (packet.data[3] & 0x40) > 0;
		 galileoE5abEnabled = (packet.data[3] & 0x80) > 0;
		 galileoE6Enabled = (packet.data[4] & 0x01) > 0;
		 qzssL1caEnabled = (packet.data[4] & 0x02) > 0;
		 qzssL1saifEnabled = (packet.data[4] & 0x04) > 0;
		 qzssL1cEnabled = (packet.data[4] & 0x08) > 0;
		 qzssL2cEnabled = (packet.data[4] & 0x10) > 0;
		 qzssL5Enabled = (packet.data[4] & 0x20) > 0;
		 qzssLexEnabled = (packet.data[4] & 0x40) > 0;
		 sbasL1caEnabled = (packet.data[4] & 0x80) > 0;
		 frequenciesCode = TypeConversion.bytesToInt64(packet.data, 1);

		 pdopMask = TypeConversion.bytesToFloat(packet.data, 9);
		 tdopMask = TypeConversion.bytesToFloat(packet.data, 13);
		 elevationMask = packet.data[17] & 0xFF;
		 snrMask = packet.data[18] & 0xFF;
		 sbasCorrectionsEnabled = packet.data[19] != 0;
		 lbandMode = packet.data[20] & 0xFF;
		 lbandFrequency = TypeConversion.bytesToUInt32(packet.data, 21);
		 lbandBaud = TypeConversion.bytesToUInt32(packet.data, 25);
		 primaryAntennaType = TypeConversion.bytesToUInt32(packet.data, 29);
		 secondaryAntennaType = TypeConversion.bytesToUInt32(packet.data, 33);
		 lbandSatelliteId = packet.data[37] & 0xFF;
	}

	public ANPacket toANPacket()
	{
		ANPacket packet = new ANPacket(LENGTH, ANPacket.PACKET_ID_GNSS_CONFIGURATION);
		packet.data[0] = 1;
		packet.data[1] |= gpsL1caEnabled ? 0x01 : 0;
		packet.data[1] |= gpsL1cEnabled ? 0x02 : 0;
		packet.data[1] |= gpsL1pEnabled ? 0x04 : 0;
		packet.data[1] |= gpsL2cEnabled ? 0x08 : 0;
		packet.data[1] |= gpsL2pEnabled ? 0x10 : 0;
		packet.data[1] |= gpsL2mEnabled ? 0x20 : 0;
		packet.data[1] |= gpsL5Enabled ? 0x40 : 0;
		packet.data[1] |= glonassG1caEnabled ? 0x80 : 0;
		packet.data[2] |= glonassG1pEnabled ? 0x01 : 0;
		packet.data[2] |= glonassL1ocEnabled ? 0x02 : 0;
		packet.data[2] |= glonassL1scEnabled ? 0x04 : 0;
		packet.data[2] |= glonassG2caEnabled ? 0x08 : 0;
		packet.data[2] |= glonassG2pEnabled ? 0x10 : 0;
		packet.data[2] |= glonassL2ocEnabled ? 0x20 : 0;
		packet.data[2] |= glonassL2scEnabled ? 0x40 : 0;
		packet.data[2] |= glonassL3ocEnabled ? 0x80 : 0;
		packet.data[3] |= glonassL3scEnabled ? 0x01 : 0;
		packet.data[3] |= beidouB1Enabled ? 0x02 : 0;
		packet.data[3] |= beidouB2Enabled ? 0x04 : 0;
		packet.data[3] |= beidouB3Enabled ? 0x08 : 0;
		packet.data[3] |= galileoE1Enabled ? 0x10 : 0;
		packet.data[3] |= galileoE5aEnabled ? 0x20 : 0;
		packet.data[3] |= galileoE5bEnabled ? 0x40 : 0;
		packet.data[3] |= galileoE5abEnabled ? 0x80 : 0;
		packet.data[4] |= galileoE6Enabled ? 0x01 : 0;
		packet.data[4] |= qzssL1caEnabled ? 0x02 : 0;
		packet.data[4] |= qzssL1saifEnabled ? 0x04 : 0;
		packet.data[4] |= qzssL1cEnabled ? 0x08 : 0;
		packet.data[4] |= qzssL2cEnabled ? 0x10 : 0;
		packet.data[4] |= qzssL5Enabled ? 0x20 : 0;
		packet.data[4] |= qzssLexEnabled ? 0x40 : 0;
		packet.data[4] |= sbasL1caEnabled ? 0x80 : 0;

		TypeConversion.floatToBytes(packet.data, 9, pdopMask);
		TypeConversion.floatToBytes(packet.data, 13, tdopMask);
		packet.data[17] = (byte) elevationMask;
		packet.data[18] = (byte) snrMask;
		packet.data[19] = (byte) (sbasCorrectionsEnabled ? 1 : 0);
		packet.data[20] = (byte) lbandMode;
		TypeConversion.int32ToBytes(packet.data, 21, (int) lbandFrequency);
		TypeConversion.int32ToBytes(packet.data, 25, (int) lbandBaud);
		TypeConversion.int32ToBytes(packet.data, 29, (int) primaryAntennaType);
		TypeConversion.int32ToBytes(packet.data, 33, (int) secondaryAntennaType);
		packet.data[37] = (byte) lbandSatelliteId;
		return packet;
	}

	public String lbandSatelliteName()
	{
		for (int i = 0; i < lBandSatelliteNumbers.length; ++i)
		{
			if (lBandSatelliteNumbers[i] == lbandSatelliteId)
			{
				return lBandSatelliteNames[i];
			}
		}
		return "";
	}

	public String lbandModeName()
	{
		for (int i = 0; i < lBandModeNumbers.length; ++i)
		{
			if (lBandModeNumbers[i] == lbandMode)
			{
				return lBandModeNames[i];
			}
		}
		return "";
	}
}
